/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.rmi.RemoteException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ejb.EJBHome;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This class works as a <strong>Business Delegate</strong> for Session
 * Enterprise Java Beans.
 * 
 * @author rlogiacco
 */
public class EnterpriseDomainBuilder extends Domain {

	/**
	 * Provides logging capabilities to the builder.
	 */
	private final static Log logger = LogFactory.getLog(EnterpriseDomainBuilder.class);

	/**
	 * Associating map for EJB homes. Home objects are cached here for single
	 * JNDI lookup, but them should be checked for bindings update.
	 */
	private final static Map homes = Collections.synchronizedMap(new HashMap());

	/**
	 * TODO documentation
	 * @param name
	 * @param type 
	 * @return
	 */
	public static Object getInstance(String name, Class type) {
		return EnterpriseDomainBuilder.getInstance(name, type, null, null);
	}

	/**
	 * TODO documentation
	 * @param name
	 * @param type 
	 * @param context
	 * @return
	 */
	public static Object getInstance(String name, Class type, Context context) {
		return EnterpriseDomainBuilder.getInstance(name, type, context, null);
	}

	/**
	 * TODO documentation
	 * @param name
	 * @param type 
	 * @param context
	 * @param parameters
	 * @return
	 */
	public static Object getInstance(String name, Class type, Context context, Object[] parameters) {
		Object home = homes.get(name);
		if (home == null) {
			try {
				if (context == null) {
					context = new InitialContext();
					logger.debug("no context specified");
				}
				home = context.lookup(name);
				if (home instanceof EJBHome) {
					home = PortableRemoteObject.narrow(home, EJBHome.class);
				}
				homes.put(name, home);
			} catch (NamingException ne) {
				logger.error("lookup(" + name + ") - error", ne);
				return null;
			}
		}
		try {
			Class[] parametersType = null;
			if (parameters != null) {
				parametersType = new Class[parameters.length];
				for (int iIndex = 0; iIndex < parametersType.length; iIndex++) {
					parametersType[iIndex] = parameters[iIndex].getClass();
				}
			}
			Object ejb = home.getClass().getDeclaredMethod("create", parametersType).invoke(home, parameters);
			return Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(), new Class[] {type}, new EnterpriseDomain(ejb));
		} catch (Exception e) {
			homes.remove(name);
			logger.error("instantiation failed", e);
			return null;
		}
	}

	private static class EnterpriseDomain implements InvocationHandler {

		/**
		 * Logger for this class
		 */
		private static final Log logger = LogFactory.getLog(EnterpriseDomain.class);

		/**
		 * TODO documentation Comment for <code>ejb</code>
		 */
		private Object ejb;

		/**
		 * TODO documentation
		 * @param ejb
		 */
		private EnterpriseDomain(Object ejb) {
			this.ejb = ejb;
		}

		/**
		 * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object,
		 *      java.lang.reflect.Method, java.lang.Object[])
		 */
		public Object invoke(Object proxy, Method method, Object[] params) throws Throwable {
			if (logger.isDebugEnabled()) {
				logger.debug("invoke(" + ejb + ", " + method + ", " + params + ")");
			}
			Object result = null;
			try {
				result = method.invoke(ejb, params);
			} catch (InvocationTargetException ite) {
				if (ite.getTargetException() instanceof RemoteException) {
					throw new BusinessException(ite);
				} else {
					throw ite.getTargetException();
				}
			}
			if (logger.isDebugEnabled()) {
				logger.debug("invoke(" + ejb + ", " + method + ", " + params + ") - finish with result=" + result);
			}
			return result;
		}
	}
}